/*{{{  #includes*/
#include <sys/types.h>
#include <sgtty.h>
#include <limits.h>
#include <signal.h>
#include <stdio.h>

#include <local/bool.h>

#define KEYBOARD_C

#include "origami.h"
/*}}}  */

/*{{{  variables*/
private struct sgttyb new,old;
private struct tchars torig_chars, twork_chars;
#ifndef NOLTCHARS
private struct ltchars ltorig_chars, ltwork_chars;
#endif

public bool echo;
public bool echo_disabled=FALSE;
private char command[PROMPT_LEN+1];
private FILE *kbd_p=NULL;
/*}}}  */

/*{{{  enable_echo*/
#ifdef STD_C
public void enable_echo(bool yes)
#else
public void enable_echo(yes) bool yes;
#endif
{
  echo=(!echo_disabled && yes);
}
/*}}}  */
/*{{{  switch_echo*/
#ifdef STD_C
public void switch_echo(bool on)
#else
public void switch_echo(on)
bool on;
#endif
{
  echo_disabled=!(echo=on);
}
/*}}}  */

/*{{{  enable_abort*/
#ifdef STD_C
public void enable_abort(void)
#else
public enable_abort()
#endif
{
  if (general_abort_key == -1) return;
  signal(SIGINT, ori_abort);
  twork_chars.t_intrc=general_abort_key;
  ioctl(fileno(stdin), TIOCSETC, &twork_chars);
}
/*}}}  */
/*{{{  disable_abort*/
#ifdef STD_C
public void disable_abort(void)
#else
public void disable_abort()
#endif
{
  signal(SIGINT, SIG_IGN);
  twork_chars.t_intrc= -1;
  ioctl(fileno(stdin), TIOCSETC, &twork_chars);
}
/*}}}  */
/*{{{  set_abort*/
#ifdef STD_C
public void set_abort(char key)
#else
public void set_abort(key) char key;
#endif
{
  general_abort_key=key;
}
/*}}}  */

/*{{{  init_keyboard*/
#ifdef STD_C
public void init_keyboard(void)
#else
public void init_keyboard()
#endif
{
  /*{{{  get old mode and set new mode*/
  /* TIOCGETP = terminal input/output control get parameters */
  ioctl(fileno(stdin), TIOCGETP, &old);
  new=old;
  new.sg_flags|=CBREAK;
  new.sg_flags&=~(ECHO|CRMOD);
  ioctl(fileno(stdin), TIOCSETP, &new);
  /*}}}  */
  /*{{{  tchars*/
  ioctl(fileno(stdin), TIOCGETC, &torig_chars);
  /* portable way to disable all keys */
  memset(&twork_chars, -1, sizeof(struct tchars));
  ioctl(fileno(stdin), TIOCSETC, &twork_chars);
  /*}}}  */
#  ifndef NOLTCHARS
  /*{{{  ltchars*/
  ioctl(fileno(stdin), TIOCGLTC, &ltorig_chars);
  memset(&ltwork_chars, -1, sizeof(struct ltchars));
  ioctl(fileno(stdin), TIOCSLTC, &ltwork_chars);
  /*}}}  */
#  endif
  disable_abort();
  enable_echo(!echo_disabled);
}
/*}}}  */
/*{{{  reset_keyboard*/
#ifdef STD_C
public void reset_keyboard(void)
#else
public void reset_keyboard()
#endif
{
  ioctl(fileno(stdin), TIOCSETP, &old);
  ioctl(fileno(stdin), TIOCSETC, &torig_chars);
#  ifndef NOLTCHARS
  ioctl(fileno(stdin), TIOCSLTC, &ltorig_chars);
#  endif
}
/*}}}  */

/*{{{  kbd_pipe*/
#ifdef STD_C
public bool kbd_pipe(FILE *p)
#else
public bool kbd_pipe(p) FILE *p;
#endif
{
  bool closed;

  if ((closed=(kbd_p!=NULL)) && (pclose(kbd_p)<0))
    err_message(M_CLOSE_FAILED,"pipe",TRUE);
  kbd_p=p;
  if ((kbd_p==NULL) && !executing_macro)
    disable_abort();
  else
    enable_abort();
  return(closed);
}
/*}}}  */

/*{{{  get_key*/
#ifdef STD_C
public TOKEN get_key (void)
#else
public TOKEN get_key ()
#endif
{
  /*{{{  variables*/
  int result= keytabcont;
  unsigned char ch=general_abort_key;
  int lg=0;
  int cp=0;
  char pread;
  /*}}}  */

  fflush(stdout);
  /*{{{  scan a keyboard-sequence 'til token is complete*/
  while (result==keytabcont && !aborted) {
    /*{{{  maybe read from pipe*/
    if (kbd_p!=NULL) {
      pread=getc(kbd_p);
      if (pread==EOF)
        kbd_pipe(NULL);
      else
        ch=pread;
    }
    /*}}}  */
    if (kbd_p==NULL)
      read(fileno(stdin), &ch, 1);
    lg++;
    result=find_key(ch);
    /*{{{  echo-handling*/
    if (echo && result==keytabcont) {
      /*{{{  append code to string*/
      if (ch>=' ') {
        /*{{{  x*/
        command[cp++]=ch;
        command[cp++]=' ';
        /*}}}  */
      } else if (ch=='\033') {
        /*{{{  M-*/
        command[cp++]='M';
        command[cp++]='-';
        /*}}}  */
      } else if (ch>=0 && ch<' ') {
        /*{{{  C-x*/
        command[cp++]='C';
        command[cp++]='-';
        command[cp++]='@'+ch;
        command[cp++]=' ';
        /*}}}  */
      } else {
        /*{{{  ?*/
        command[cp++]='?';
        command[cp++]=' ';
        /*}}}  */
      }
      command[cp]='\0';
      /*}}}  */
      message(command,FALSE);
    }
    /*}}}  */
  }
  /*}}}  */
  if (aborted) return(O_BREAK);
  if (result!=keytabend)
    return (result);
  else if (ch==general_abort_key) {
    aborted=TRUE;
    return(O_BREAK);
  } else if (lg==1)
    return(ch);
  if (echo) message(M_NOT_BOUND, TRUE);
  return(O_NOP);
}
/*}}}  */
